'''
@author: sean
'''

from setuptools import setup, find_packages

setup(
    name='rpmfile',
    description='Read rmp archive files',
    version="0.1",
    author='Sean Ross-Ross',
    author_email='srossross@gmail.com',
    packages=find_packages(),

)
